@@ -17,6 +17,7 @@
 
 int test_fmt_pl(void)
 {
+	int err;
 	const struct pl pl   = PL("rattarei");
 	const struct pl pl0  = PL("rattarei");
 	const struct pl pl0_ = PL("rAtTaReI");
@@ -94,6 +95,16 @@ int test_fmt_pl(void)
 	if (0 == pl_strcmp(&pl3, str0))
 		goto out;
 
+	/* pl_strncmp() */
+	err = pl_strncmp(&pl, "rat", 3);
+	TEST_ERR(err);
+	err = pl_strncmp(&pl, "RAT", 3);
+	TEST_EQUALS(EINVAL, err);
+
+	/* pl_strncasecmp() */
+	err = pl_strncasecmp(&pl, "RaT", 3);
+	TEST_ERR(err);
+
 	/* pl_strcasecmp() */
 	if (EINVAL != pl_strcasecmp(NULL, NULL))
 		goto out;
@@ -1056,6 +1067,12 @@ int test_fmt_str_bool(void)
 		err = str_bool(&en, truestr[i]);
 		TEST_ERR(err);
 		TEST_EQUALS(true, en);
+
+		struct pl s;
+		pl_set_str(&s, truestr[i]);
+		err = pl_bool(&en, &s);
+		TEST_ERR(err);
+		TEST_EQUALS(true, en);
 	}
 
 	state = STATE_FALSE;
@@ -1063,12 +1080,23 @@ int test_fmt_str_bool(void)
 		err = str_bool(&en, falsestr[i]);
 		TEST_ERR(err);
 		TEST_EQUALS(false, en);
+
+		struct pl s;
+		pl_set_str(&s, falsestr[i]);
+		err = pl_bool(&en, &s);
+		TEST_ERR(err);
+		TEST_EQUALS(false, en);
 	}
 
 	state = STATE_UNSUP;
 	for (i = 0; i < RE_ARRAY_SIZE(notsup); i++) {
 		err = str_bool(&en, notsup[i]);
 		TEST_EQUALS(err, EINVAL);
+
+		struct pl s;
+		pl_set_str(&s, notsup[i]);
+		err = pl_bool(&en, &s);
+		TEST_EQUALS(err, EINVAL);
 	}
 
 	err = 0;
@@ -1185,3 +1213,80 @@ out:
 	mem_deref(mb);
 	return err;
 }
+
+
+int test_fmt_trim(void)
+{
+	const struct pl pla = PL("  heisann  ");
+	const struct pl pll = PL("heisann  ");
+	const struct pl plr = PL("  heisann");
+	const struct pl plb = PL("heisann");
+
+	const struct pl pl  = PL("	 \r	\n  heisann	\n \r  ");
+	const struct pl pl1 = PL("heisann	\n \r  ");
+	const struct pl pl2 = PL("	 \r	\n  heisann");
+	const struct pl pl3 = PL("heisann");
+
+	const struct pl pl4 = PL("	 \r \n");
+	const struct pl pl5 = PL("");
+	struct pl pln;
+
+	int err = 0;
+
+	pln = pla;
+	err = pl_ltrim(&pln);
+	TEST_ERR(err);
+	err = pl_cmp(&pln, &pll);
+	TEST_ERR(err);
+
+	pln = pla;
+	err = pl_rtrim(&pln);
+	TEST_ERR(err);
+	err = pl_cmp(&pln, &plr);
+	TEST_ERR(err);
+
+	pln = pla;
+	err = pl_trim(&pln);
+	TEST_ERR(err);
+	err = pl_cmp(&pln, &plb);
+	TEST_ERR(err);
+
+	pln = pl1;
+	err = pl_ltrim(&pln);
+	TEST_ERR(err);
+	err = pl_cmp(&pln, &pl1);
+	TEST_ERR(err);
+
+	pln = pl;
+	err = pl_rtrim(&pln);
+	TEST_ERR(err);
+	err = pl_cmp(&pln, &pl2);
+	TEST_ERR(err);
+
+	pln = pl;
+	err = pl_trim(&pln);
+	TEST_ERR(err);
+	err = pl_cmp(&pln, &pl3);
+	TEST_ERR(err);
+
+	pln = pl4;
+	err = pl_ltrim(&pln);
+	TEST_ERR(err);
+	err = pl_cmp(&pln, &pl5);
+	TEST_ERR(err);
+
+	pln = pl4;
+	err = pl_rtrim(&pln);
+	TEST_ERR(err);
+	err = pl_cmp(&pln, &pl5);
+	TEST_ERR(err);
+
+	pln = pl4;
+	err = pl_trim(&pln);
+	TEST_ERR(err);
+	err = pl_cmp(&pln, &pl5);
+	TEST_ERR(err);
+
+out:
+	return err;
+}
