@@ -52,6 +52,7 @@ static const struct test tests[] = {
 	TEST(test_aac),
 	TEST(test_aes),
 	TEST(test_aes_gcm),
+	TEST(test_au),
 	TEST(test_aubuf),
 	TEST(test_aulength),
 	TEST(test_aulevel),
@@ -59,11 +60,13 @@ static const struct test tests[] = {
 	TEST(test_auresamp),
 	TEST(test_async),
 	TEST(test_av1),
+	TEST(test_dd),
 	TEST(test_base64),
 	TEST(test_bfcp),
 	TEST(test_bfcp_bin),
-	TEST(test_bfcp_udp),
 	TEST(test_bfcp_tcp),
+	TEST(test_bfcp_udp),
+	TEST(test_btrace),
 	TEST(test_conf),
 	TEST(test_crc32),
 	TEST(test_dns_hdr),
@@ -118,6 +121,8 @@ static const struct test tests[] = {
 	TEST(test_https_loop),
 	TEST(test_http_client_set_tls),
 	TEST(test_https_large_body),
+#endif
+#ifdef HAVE_TLS1_3_POST_HANDSHAKE_AUTH
 	TEST(test_https_conn_post_handshake),
 #endif
 	TEST(test_httpauth_chall),
@@ -140,12 +145,14 @@ static const struct test tests[] = {
 	TEST(test_mbuf),
 	TEST(test_md5),
 	TEST(test_mem),
+	TEST(test_mem_pool),
 	TEST(test_mem_reallocarray),
 	TEST(test_mem_secure),
 	TEST(test_net_if),
 	TEST(test_mqueue),
 	TEST(test_odict),
 	TEST(test_odict_array),
+	TEST(test_odict_pl),
 	TEST(test_pcp),
 	TEST(test_remain),
 	TEST(test_re_assert_se),
@@ -162,6 +169,8 @@ static const struct test tests[] = {
 	TEST(test_rtcp_decode_badmsg),
 	TEST(test_rtcp_packetloss),
 	TEST(test_rtcp_twcc),
+	TEST(test_rtcp_xr),
+	TEST(test_rtcp_loop),
 	TEST(test_sa_class),
 	TEST(test_sa_cmp),
 	TEST(test_sa_decode),
@@ -174,6 +183,7 @@ static const struct test tests[] = {
 	TEST(test_sdp_oa),
 	TEST(test_sdp_extmap),
 	TEST(test_sdp_disabled_rejected),
+	TEST(test_sdp_interop),
 	TEST(test_sha1),
 	TEST(test_sip_addr),
 	TEST(test_sip_auth),
@@ -213,8 +223,10 @@ static const struct test tests[] = {
 	TEST(test_sys_fs_fopen),
 	TEST(test_sys_getenv),
 	TEST(test_tcp),
+	TEST(test_tcp_tos),
 	TEST(test_telev),
 	TEST(test_text2pcap),
+	TEST(test_fmt_trim),
 #ifdef USE_TLS
 	TEST(test_tls),
 	TEST(test_tls_ec),
@@ -233,6 +245,7 @@ static const struct test tests[] = {
 	TEST(test_turn),
 	TEST(test_turn_tcp),
 	TEST(test_udp),
+	TEST(test_udp_tos),
 	TEST(test_unixsock),
 	TEST(test_uri),
 	TEST(test_uri_encode),
@@ -247,6 +260,7 @@ static const struct test tests[] = {
 	TEST(test_websock),
 	TEST(test_trace),
 	TEST(test_thread),
+	TEST(test_thread_tss),
 
 #ifdef USE_TLS
 	/* combination tests: */
@@ -272,6 +286,7 @@ static const struct test tests_integration[] = {
 	TEST(test_tmr_jiffies_usec),
 	TEST(test_turn_thread),
 	TEST(test_thread_cnd_timedwait),
+	TEST(test_cplusplus),
 };
 
 
@@ -378,6 +393,35 @@ static const struct test *find_test_int(const char *name)
 }
 
 
+static int test_exec(const struct test *test)
+{
+	if (!test)
+		return EINVAL;
+
+	struct memstat mstat_before;
+	struct memstat mstat_after;
+
+	mem_get_stat(&mstat_before);
+
+	int err = test->exec();
+	re_fhs_flush();
+
+	mem_get_stat(&mstat_after);
+
+	if (mstat_after.blocks_cur > mstat_before.blocks_cur) {
+		mem_debug();
+		re_assert(false && "Test leaks memory blocks");
+	}
+
+	if (mstat_after.bytes_cur > mstat_before.bytes_cur) {
+		mem_debug();
+		re_assert(false && "Test leaks memory bytes");
+	}
+
+	return err;
+}
+
+
 /**
  * Run a single testcase in OOM (Out-of-memory) mode.
  *
@@ -409,7 +453,7 @@ static int testcase_oom(const struct test *test, int levels, bool verbose)
 
 		mem_threshold_set(i);
 
-		err = test->exec();
+		err = test_exec(test);
 		if (err == 0) {
 			/* success, stop now */
 			break;
@@ -450,7 +494,7 @@ static int testcase_oom(const struct test *test, int levels, bool verbose)
 int test_oom(const char *name, bool verbose)
 {
 	size_t i;
-	const int levels = 64;
+	const int levels = 128;
 	int err = 0;
 
 	test_mode = TEST_MEMORY;
@@ -513,7 +557,7 @@ static int test_unit(const char *name, bool verbose)
 			goto out;
 		}
 
-		err = test->exec();
+		err = test_exec(test);
 		if (err) {
 			DEBUG_WARNING("%s: test failed (%m)\n", name, err);
 			goto out;
@@ -586,7 +630,7 @@ static int testcase_perf(const struct test *test, double *usec_avgp)
 	usec_start = tmr_jiffies_usec();
 	for (i = 1; i <= DRYRUN_MAX; i++) {
 
-		err = test->exec();
+		err = test_exec(test);
 		if (err)
 			return err;
 
@@ -604,7 +648,7 @@ static int testcase_perf(const struct test *test, double *usec_avgp)
 	/* now for the real measurement */
 	usec_start = tmr_jiffies_usec();
 	for (i=0; i<n; i++) {
-		err = test->exec();
+		err = test_exec(test);
 		if (err)
 			return err;
 	}
@@ -843,7 +887,7 @@ int test_multithread(void)
 	for (i=0; i<RE_ARRAY_SIZE(threadv); i++) {
 
 		if (threadv[i].err != 0) {
-			re_printf("%u failed: %-30s  [%d] [%m]\n", i,
+			re_printf("%zu failed: %-30s  [%d] [%m]\n", i,
 				  threadv[i].test->name,
 				  threadv[i].err, threadv[i].err);
 			err = threadv[i].err;
@@ -1107,7 +1151,7 @@ int test_integration(const char *name, bool verbose)
 		if (str_isset(name) && test->name)
 			continue;
 
-		(void)re_fprintf(stderr, "  %-28s: ", test->name);
+		(void)re_fprintf(stderr, "  %-32s: ", test->name);
 
 		if (test->exec)
 			err = test->exec();
